# frozen_string_literal: true

require 'json'

module Aries
  class Character
    attr_reader :id, :name, :job, :world, :level, :paragon, :fame

    def initialize(attributes)
      @id      = attributes[:id]
      @name    = attributes[:name]
      @job     = attributes[:job]
      @world   = attributes[:world] || 'Scania'
      @level   = attributes[:level]
      @paragon = attributes[:paragon]
      @fame    = attributes[:fame]
    end

    def to_json
      hash = {}
      instance_variables.each do |var|
        var = var[1..-1]
        hash[var] = send(var.to_sym)
      end

      JSON.pretty_generate(hash)
    end

    def to_hash
      JSON.parse(to_json, symbolize_names: true)
    end

    def self.online
      online = Aries::Scraper.raw(page: 'home')
                             .body
                             .match(/(\d+)\sONLINE/)

      return false unless online

      online.captures.first.to_i
    end

    def self.all(n = 1)
      Aries::Scraper.get(page: 'ranking', overall: skip(n))
                    .map { |attr| new(attr) }
    end

    def self.fame(n = 1)
      Aries::Scraper.get(page: 'ranking', fame: 1, start: skip(n))
                    .map { |attr| new(attr) }
    end

    def self.job(name, n = 1)
      Aries::Scraper.get(page: 'ranking', job: job_id(name), start: skip(n))
                    .map { |attr| new(attr) }
    end

    def self.find(ign)
      new Aries::Scraper.post({ page: 'ranking', name: 1 }, username: ign)
                        .first
    end

    class << self
      protected

      # HACK: Pagination
      # Skip <number> of characters
      def skip(page)
        (page * 10) - 10
      end

      def job_id(name)
        file = File.read \
          File.expand_path('../../../../data/classes.json', __FILE__)
        job = JSON.parse(file).find { |j| j['name'] == name }

        raise Aries::Exception::InvalidJobError, 'Invalid job' if job.nil?

        job['id']
      end
    end
  end
end
